import oracledb from 'oracledb';
import { initializeEnvironment } from './Util/Environment.js';

oracledb.fetchAsString = [ oracledb.CLOB, oracledb.DATE ];
initializeEnvironment('../environment.json');
oracledb.initOracleClient({ libDir: process.env.LD_LIBRARY_PATH });
// console.log(process.env.UV_THREADPOOL_SIZE);
oracledb.fetchArraySize = 10000;
const BD = {
    pool: null,
    conn: null,
    usePool: false,
    connections: [],
    getOracleConnectionString(host, port, service){
        return `${host}:${port}/${service}`;
    },
    close( conn ){
        conn.close();
    },
    closeAll(){
        //this.connections.forEach(this.close);
        if(this.pool){
            this.pool.close(0);
        }
    },
    async getSimpleConnection(){
        const {
            DB_ORA_USER,
            DB_ORA_PASSWORD,
            DB_ORA_HOST,
            DB_ORA_PORT,
            DB_ORA_DBNAME
        } = process.env;
        let connection = await oracledb.getConnection({
            user: DB_ORA_USER,
            password: DB_ORA_PASSWORD,
            connectString: this.getOracleConnectionString(DB_ORA_HOST, DB_ORA_PORT, DB_ORA_DBNAME)
        });
        this.connections.push(connection);
        return connection;
    },
    async getConnection(){
        let connection = null;
        if(this.usePool){
            this.pool = this.pool == null ? await oracledb.createPool({
                user: process.env.DB_ORA_USER,
                password: process.env.DB_ORA_PASSWORD,
                connectString: this.getOracleConnectionString(process.env.DB_ORA_HOST, process.env.DB_ORA_PORT, process.env.DB_ORA_DBNAME),
                poolMin         : 10,
                poolMax         : 10,
                poolTimeout     : 60,
                poolIncrement : 0

            }) : this.pool;
            connection = await this.pool.getConnection();
            this.connections.push(connection);
        } else {
            connection = this.conn ? this.conn : await this.getSimpleConnection();
            this.conn = !this.conn ? connection : this.conn;
        }
        return connection;
    },
    resolveParamType(type){
        const TYPES = {
            STRING: oracledb.STRING,
            NUMBER: oracledb.NUMBER,
        };
        return type in TYPES ? TYPES[type] : oracledb[type] || null;
    },
    resolveDirectionType( direction ){
        const DIRECTIONS = {
            IN: oracledb.BIND_IN,
            OUT: oracledb.BIND_OUT,
            INOUT: oracledb.BIND_INOUT
        };
        return DIRECTIONS[direction] || DIRECTIONS.IN;
    },
    createParam(name, value, dir = 'IN', type = null, maxSize = null){
        let PARAM = {};
        PARAM[name] = {
            dir:  this.resolveDirectionType(dir),
            type: this.resolveParamType( type ),
            val: value
        };
        if(PARAM[name].type === null) delete PARAM[name].type;
        if([oracledb.BIND_OUT, oracledb.BIND_INOUT].includes(PARAM[name].dir) && maxSize !== null){
            PARAM[name].maxSize = maxSize;
        }
        return PARAM;
    },
    async execQuery( SQL, params = {}){
        let conn = await this.getConnection();
        const result = await conn.execute(
            SQL,
            params,
            {
                outFormat: oracledb.OUT_FORMAT_OBJECT
            }
        );
        return result.rows;
    },
    async execSQL(SQL, params = {}){
        let conn = await this.getConnection();
        const result = await conn.execute(
            SQL,
            params
        );
        conn.commit();
        return result;
    },
    async execProc(name, params){
        try {
            let conn = await this.getConnection();
            const procedurePrototype = `BEGIN
                ${name}(:${Object.keys(params).join(', :')});
            END;`;
            const result = await conn.execute(
                procedurePrototype,
                params
            );
            conn.commit();
            if(result.outBinds) return result.outBinds;
            return result;
        } catch ( error ){
            throw new Error(error.message);
        }
    }
}


export default BD;